@@ -74,6 +74,11 @@ send_error_page(client_t *client)
     int status = client->bad_request_code;
     int r = status < 0 ? status * -1:status;
     client->status_code = r;
+
+#ifdef DEBUG
+    printf("send_error_page status_code %d client %p \n", status, client);
+#endif
+
     switch(r){
         case 400:
             blocking_write(client, MSG_400, sizeof(MSG_400) -1);
@@ -148,7 +153,7 @@ static inline int
 writev_bucket(write_bucket *data)
 {
     size_t w;
-    int i = 0;
+    register int i = 0;
     w = writev(data->fd, data->iov, data->iov_cnt);
     if(w == -1){
         //error
@@ -227,9 +232,9 @@ write_headers(client_t *client, char *data, size_t datalen)
         return 1;
     }
     write_bucket *bucket; 
-    uint32_t i = 0, hlen = 0;
-    PyObject *headers = NULL;
-    PyObject *object = NULL;
+    register uint32_t i = 0, hlen = 0;
+    register PyObject *headers = NULL;
+    register PyObject *object = NULL;
     char *name = NULL;
     Py_ssize_t namelen;
     char *value = NULL;
@@ -282,8 +287,8 @@ write_headers(client_t *client, char *data, size_t datalen)
                 goto error;
             }
             
-            object1 = PyTuple_GetItem(tuple, 0);
-            object2 = PyTuple_GetItem(tuple, 1);
+            object1 = PyTuple_GET_ITEM(tuple, 0);
+            object2 = PyTuple_GET_ITEM(tuple, 1);
             
             if (PyString_Check(object1)) {
                 PyString_AsStringAndSize(object1, &name, &namelen);
@@ -303,6 +308,13 @@ write_headers(client_t *client, char *data, size_t datalen)
                 goto error;
             }
 
+            if (strchr(name, ':') != 0) {
+                PyErr_Format(PyExc_ValueError, "header name may not contains ':'"
+                             "response header with name '%s' and value '%s'",
+                             name, value);
+                goto error;
+            }
+
             if (strchr(name, '\n') != 0 || strchr(value, '\n') != 0) {
                 PyErr_Format(PyExc_ValueError, "embedded newline in "
                              "response header with name '%s' and value '%s'",
@@ -417,7 +429,7 @@ close_response(client_t *client)
 
 
 static inline int
-processs_sendfile(client_t *client)
+processs_sendfile(register client_t *client)
 {
     PyObject *filelike = NULL;
     FileWrapperObject *filewrap = NULL;
@@ -465,13 +477,13 @@ processs_sendfile(client_t *client)
 }
 
 static inline int
-processs_write(client_t *client)
+processs_write(register client_t *client)
 {
-    PyObject *iterator = NULL;
+    register PyObject *iterator = NULL;
     register PyObject *item;
     char *buf;
     Py_ssize_t buflen;
-    write_bucket *bucket;
+    register write_bucket *bucket;
     int ret;
 
     iterator = client->response_iter;
@@ -557,6 +569,9 @@ start_response_file(client_t *client)
     in_fd = PyObject_AsFileDescriptor(filelike);
     if (in_fd == -1) {
         PyErr_Clear();
+#ifdef DEBUG
+        printf("can't get fd \n");
+#endif
         return -1;
     }
     if (fstat(in_fd, &info) == -1){
@@ -594,16 +609,21 @@ start_response_write(client_t *client)
         buflen = PyString_GET_SIZE(item);
 
 #ifdef DEBUG
-        printf("start_response_write buflen %d \n", buflen);
+        printf("start_response_write status_code %d buflen %d \n", client->status_code, buflen);
 #endif
         Py_DECREF(item);
         return write_headers(client, buf, buflen);
     }else{
-        PyErr_SetString(PyExc_TypeError, "response item must be a string");
-        Py_XDECREF(item);
-        if (PyErr_Occurred()){ 
-            write_error_log(__FILE__, __LINE__);
-            return -1;
+        if (item == NULL && !PyErr_Occurred()){ 
+            //Stap Iteration
+            return write_headers(client, NULL, 0);
+        }else{
+            PyErr_SetString(PyExc_TypeError, "response item must be a string");
+            Py_XDECREF(item);
+            if (PyErr_Occurred()){ 
+                write_error_log(__FILE__, __LINE__);
+                return -1;
+            }
         }
         
     }
@@ -619,6 +639,9 @@ response_start(client_t *client)
         return write_headers(client, NULL, 0);
     }
     if (CheckFileWrapper(client->response)) {
+#ifdef DEBUG
+        printf("use sendfile \n");
+#endif 
         ret = start_response_file(client);
         if(ret > 0){
             // sended header 
@@ -627,7 +650,7 @@ response_start(client_t *client)
     }else{
         ret = start_response_write(client);
 #ifdef DEBUG
-        printf("start_response_write ret = %d \n", ret);
+        printf("start_response_write status_code %d ret = %d \n", client->status_code, ret);
 #endif 
         if(ret > 0){
             // sended header 
@@ -737,6 +760,11 @@ ResponseObject_call(PyObject *obj, PyObject *args, PyObject *kw)
         return NULL;
     }
 
+    if (int_code < 100 || int_code > 999) {
+        PyErr_SetString(PyExc_ValueError, "status code is invalid");
+        return NULL;
+    }
+
     self->cli->status_code = int_code;
 
     Py_XDECREF(self->cli->headers);
@@ -772,6 +800,10 @@ FileWrapperObject_iter(PyObject *o)
         PyErr_SetString(PyExc_TypeError, "file-like object must be a iterable object");
         return NULL;
     }
+
+#ifdef DEBUG
+    printf("use FileWrapperObject_iter \n");
+#endif 
     return (PyObject *)iterator;
 }
 
@@ -817,9 +849,22 @@ file_wrapper(PyObject *self, PyObject *args)
 inline int 
 CheckFileWrapper(PyObject *obj)
 {
+    FileWrapperObject *f;
+    PyObject *filelike;
+    int in_fd;
     if (obj->ob_type != &FileWrapperType){
         return 0;
     }
+
+    f = (FileWrapperObject *)obj;
+    filelike = f->filelike;
+
+    in_fd = PyObject_AsFileDescriptor(filelike);
+    if (in_fd == -1) {
+        PyErr_Clear();
+        return 0;
+    }
+
     return 1;
 }
 
