@@ -16,8 +16,6 @@
 
 #define ACCEPT_TIMEOUT_SECS 1
 #define SHORT_TIMEOUT_SECS 2
-#define WRITE_TIMEOUT_SECS 60 * 5 
-//#define READ_LONG_TIMEOUT_SECS 3
 
 #define MAX_BUFSIZE 1024 * 8
 #define INPUT_BUF_SIZE 1024 * 8
@@ -54,6 +52,15 @@ PyObject* hub_switch_value;
 PyObject* current_client;
 PyObject* timeout_error;
 
+/* reuse object */
+static PyObject *client_key = NULL; //meinheld.client
+static PyObject *wsgi_input_key = NULL; //wsgi.input key
+static PyObject *empty_string = NULL; //""
+
+/* gunicorn */
+static int spinner = 0;
+static int tempfile_fd = 0;
+static int ppid = 0;
 
 static void
 r_callback(picoev_loop* loop, int fd, int events, void* cb_arg);
@@ -109,7 +116,7 @@ clean_cli(client_t *client)
     
     Py_CLEAR(client->response);
 #ifdef DEBUG
-    printf("close environ %p \n", client->environ);
+    printf("clean_cli environ status_code %d address %p \n", client->status_code, client->environ);
 #endif
     
     // force clear
@@ -147,7 +154,7 @@ close_conn(client_t *cli, picoev_loop* loop)
     if(!cli->keep_alive){
         close(cli->fd);
 #ifdef DEBUG
-        printf("close client:%p fd:%d \n", cli, cli->fd);
+        printf("close client:%p fd:%d status_code %d \n", cli, cli->fd, cli->status_code);
 #endif
     }else{
         disable_cork(cli);
@@ -224,7 +231,7 @@ process_wsgi_app(client_t *cli)
     }
     args = Py_BuildValue("(OO)", cli->environ, start);
 
-    current_client = PyDict_GetItemString(cli->environ, "meinheld.client");
+    current_client = PyDict_GetItem(cli->environ, client_key);
     pyclient = (ClientObject *)current_client;
 
 #ifdef DEBUG
@@ -360,7 +367,6 @@ w_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
     
     } else if ((events & PICOEV_WRITE) != 0) {
         ret = process_body(client);
-        picoev_set_timeout(loop, client->fd, WRITE_TIMEOUT_SECS);
 #ifdef DEBUG
         printf("process_body ret %d \n", ret);
 #endif
@@ -414,7 +420,7 @@ resume_wsgi_app(ClientObject *pyclient, picoev_loop* loop)
 #endif
             //clear event
             picoev_del(loop, client->fd);
-            picoev_add(loop, client->fd, PICOEV_WRITE, WRITE_TIMEOUT_SECS, w_callback, (void *)client);
+            picoev_add(loop, client->fd, PICOEV_WRITE, 0, w_callback, (void *)client);
             return;
         default:
             // send OK
@@ -427,6 +433,10 @@ call_wsgi_app(client_t *client, picoev_loop* loop)
 {
     int ret;
     ret = process_wsgi_app(client);
+
+#ifdef DEBUG
+    printf("call_wsgi_app result %d \n", ret);
+#endif
     switch(ret){
         case -1:
             //Internal Server Error
@@ -448,6 +458,9 @@ call_wsgi_app(client_t *client, picoev_loop* loop)
         return;
     }
     ret = response_start(client);
+#ifdef DEBUG
+    printf("response_start result %d \n", ret);
+#endif
     switch(ret){
         case -1:
             // Internal Server Error
@@ -463,7 +476,7 @@ call_wsgi_app(client_t *client, picoev_loop* loop)
 #endif
             //clear event
             picoev_del(loop, client->fd);
-            picoev_add(loop, client->fd, PICOEV_WRITE, WRITE_TIMEOUT_SECS, w_callback, (void *)client);
+            picoev_add(loop, client->fd, PICOEV_WRITE, 0, w_callback, (void *)client);
             return;
         default:
             // send OK
@@ -484,7 +497,7 @@ prepare_call_wsgi(client_t *client)
         fflush(tmp);
         rewind(tmp);
         input = PyFile_FromFile(tmp, "<tmpfile>", "r", fclose);
-        PyDict_SetItemString((PyObject *)client->environ, "wsgi.input", input);
+        PyDict_SetItem((PyObject *)client->environ, wsgi_input_key, input);
         Py_DECREF(input);
         client->body = NULL;
     }else{
@@ -492,15 +505,16 @@ prepare_call_wsgi(client_t *client)
             object = PycStringIO->cgetvalue((PyObject *)client->body);
             Py_XDECREF((PyObject *)client->body);
             input = PycStringIO->NewInput(object);
-            PyDict_SetItemString((PyObject *)client->environ, "wsgi.input", input);
+            PyDict_SetItem((PyObject *)client->environ, wsgi_input_key, input);
         }else{
             if(client->body){
                 object = client->body;
             }else{
-                object = PyString_FromString("");
+                object = empty_string;
+                Py_INCREF(empty_string);
             }
             input = PycStringIO->NewInput(object);
-            PyDict_SetItemString((PyObject *)client->environ, "wsgi.input", input);
+            PyDict_SetItem((PyObject *)client->environ, wsgi_input_key, input);
         }
         client->body = object;
         //Py_DECREF(object);
@@ -691,7 +705,11 @@ setup_server_env(void)
     setup_client();
     PycString_IMPORT;
     PyGreenlet_Import();
+    
     hub_switch_value = Py_BuildValue("(i)", -1);
+    client_key = PyString_FromString("meinheld.client");
+    wsgi_input_key = PyString_FromString("wsgi.input");
+    empty_string = PyString_FromString("");
 }
 
 static inline int 
@@ -815,6 +833,17 @@ unix_listen(char *sock_name)
     return 1;
 }
 
+static inline void
+fast_notify(void)
+{
+    spinner = (spinner + 1) % 2;
+    fchmod(tempfile_fd, spinner);
+    if(ppid != getppid()){
+        loop_done = 0;
+        tempfile_fd = 0;
+    }
+}
+
 static PyObject *
 meinheld_listen(PyObject *self, PyObject *args)
 {
@@ -952,7 +981,9 @@ meinheld_run_loop(PyObject *self, PyObject *args)
         picoev_loop_once(main_loop, 10);
         i++;
         // watchdog slow.... skip check
-        if(watchdog && i > 15){
+        
+        //if(watchdog && i > 1){
+        if(watchdog){
             watchdog_result = PyObject_CallFunction(watchdog, NULL);
             if(PyErr_Occurred()){
                 PyErr_Print();
@@ -960,6 +991,8 @@ meinheld_run_loop(PyObject *self, PyObject *args)
             }
             Py_XDECREF(watchdog_result);
             i = 0;
+        }else if(tempfile_fd){
+            fast_notify();
         }
 #ifdef DEBUG
         //printf("loop \n");
@@ -977,6 +1010,9 @@ meinheld_run_loop(PyObject *self, PyObject *args)
     clear_static_env();
 
     Py_DECREF(hub_switch_value);
+    Py_DECREF(client_key);
+    Py_DECREF(wsgi_input_key);
+    Py_DECREF(empty_string);
 
     if(unix_sock_name){
         unlink(unix_sock_name);
@@ -1085,6 +1121,19 @@ meinheld_set_listen_socket(PyObject *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyObject *
+meinheld_set_fastwatchdog(PyObject *self, PyObject *args)
+{
+    int _fd;
+    int _ppid;
+    if (!PyArg_ParseTuple(args, "ii", &_fd, &_ppid))
+        return NULL;
+    
+    tempfile_fd = _fd;
+    ppid = _ppid;
+    Py_RETURN_NONE;
+}
+
 PyObject *
 meinheld_set_watchdog(PyObject *self, PyObject *args)
 {
@@ -1320,6 +1369,8 @@ meinheld_trampolin(PyObject *self, PyObject *args, PyObject *kwargs)
 }
 
 
+
+
 static PyMethodDef WsMethods[] = {
     {"listen", meinheld_listen, METH_VARARGS, "set host and port num"},
     {"access_log", meinheld_access_log, METH_VARARGS, "set access log file path."},
@@ -1342,13 +1393,14 @@ static PyMethodDef WsMethods[] = {
     // support gunicorn 
     {"set_listen_socket", meinheld_set_listen_socket, METH_VARARGS, "set listen_sock"},
     {"set_watchdog", meinheld_set_watchdog, METH_VARARGS, "set watchdog"},
+    {"set_fastwatchdog", meinheld_set_fastwatchdog, METH_VARARGS, "set watchdog"},
     {"run", meinheld_run_loop, METH_VARARGS, "set wsgi app, run the main loop"},
     // greenlet and continuation
     {"_suspend_client", meinheld_suspend_client, METH_VARARGS, "resume client"},
     {"_resume_client", meinheld_resume_client, METH_VARARGS, "resume client"},
     // io
     {"cancel_wait", meinheld_cancel_wait, METH_VARARGS, "cancel wait"},
-    {"trampolin", meinheld_trampolin, METH_VARARGS | METH_KEYWORDS, "trampolin"},
+    {"trampolin", (PyCFunction)meinheld_trampolin, METH_VARARGS | METH_KEYWORDS, "trampolin"},
 
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
